diff -pruN 0.20.1-1/CHANGELOG.md 0.20.2-1/CHANGELOG.md
--- 0.20.1-1/CHANGELOG.md	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/CHANGELOG.md	2024-03-14 19:49:11.000000000 +0000
@@ -1,5 +1,114 @@
 # Changelog
 
+## 0.20.2
+
+* Support TypeScript experimental decorators on `abstract` class fields ([#3684](https://github.com/evanw/esbuild/issues/3684))
+
+    With this release, you can now use TypeScript experimental decorators on `abstract` class fields. This was silently compiled incorrectly in esbuild 0.19.7 and below, and was an error from esbuild 0.19.8 to esbuild 0.20.1. Code such as the following should now work correctly:
+
+    ```ts
+    // Original code
+    const log = (x: any, y: string) => console.log(y)
+    abstract class Foo { @log abstract foo: string }
+    new class extends Foo { foo = '' }
+
+    // Old output (with --loader=ts --tsconfig-raw={\"compilerOptions\":{\"experimentalDecorators\":true}})
+    const log = (x, y) => console.log(y);
+    class Foo {
+    }
+    new class extends Foo {
+      foo = "";
+    }();
+
+    // New output (with --loader=ts --tsconfig-raw={\"compilerOptions\":{\"experimentalDecorators\":true}})
+    const log = (x, y) => console.log(y);
+    class Foo {
+    }
+    __decorateClass([
+      log
+    ], Foo.prototype, "foo", 2);
+    new class extends Foo {
+      foo = "";
+    }();
+    ```
+
+* JSON loader now preserves `__proto__` properties ([#3700](https://github.com/evanw/esbuild/issues/3700))
+
+    Copying JSON source code into a JavaScript file will change its meaning if a JSON object contains the `__proto__` key. A literal `__proto__` property in a JavaScript object literal sets the prototype of the object instead of adding a property named `__proto__`, while a literal `__proto__` property in a JSON object literal just adds a property named `__proto__`. With this release, esbuild will now work around this problem by converting JSON to JavaScript with a computed property key in this case:
+
+    ```js
+    // Original code
+    import data from 'data:application/json,{"__proto__":{"fail":true}}'
+    if (Object.getPrototypeOf(data)?.fail) throw 'fail'
+
+    // Old output (with --bundle)
+    (() => {
+      // <data:application/json,{"__proto__":{"fail":true}}>
+      var json_proto_fail_true_default = { __proto__: { fail: true } };
+
+      // entry.js
+      if (Object.getPrototypeOf(json_proto_fail_true_default)?.fail)
+        throw "fail";
+    })();
+
+    // New output (with --bundle)
+    (() => {
+      // <data:application/json,{"__proto__":{"fail":true}}>
+      var json_proto_fail_true_default = { ["__proto__"]: { fail: true } };
+
+      // example.mjs
+      if (Object.getPrototypeOf(json_proto_fail_true_default)?.fail)
+        throw "fail";
+    })();
+    ```
+
+* Improve dead code removal of `switch` statements ([#3659](https://github.com/evanw/esbuild/issues/3659))
+
+    With this release, esbuild will now remove `switch` statements in branches when minifying if they are known to never be evaluated:
+
+    ```js
+    // Original code
+    if (true) foo(); else switch (bar) { case 1: baz(); break }
+
+    // Old output (with --minify)
+    if(1)foo();else switch(bar){case 1:}
+
+    // New output (with --minify)
+    foo();
+    ```
+
+* Empty enums should behave like an object literal ([#3657](https://github.com/evanw/esbuild/issues/3657))
+
+    TypeScript allows you to create an empty enum and add properties to it at run time. While people usually use an empty object literal for this instead of a TypeScript enum, esbuild's enum transform didn't anticipate this use case and generated `undefined` instead of `{}` for an empty enum. With this release, you can now use an empty enum to generate an empty object literal.
+
+    ```ts
+    // Original code
+    enum Foo {}
+
+    // Old output (with --loader=ts)
+    var Foo = /* @__PURE__ */ ((Foo2) => {
+    })(Foo || {});
+
+    // New output (with --loader=ts)
+    var Foo = /* @__PURE__ */ ((Foo2) => {
+      return Foo2;
+    })(Foo || {});
+    ```
+
+* Handle Yarn Plug'n'Play edge case with `tsconfig.json` ([#3698](https://github.com/evanw/esbuild/issues/3698))
+
+    Previously a `tsconfig.json` file that `extends` another file in a package with an `exports` map failed to work when Yarn's Plug'n'Play resolution was active. This edge case should work now starting with this release.
+
+* Work around issues with Deno 1.31+ ([#3682](https://github.com/evanw/esbuild/issues/3682))
+
+    Version 0.20.0 of esbuild changed how the esbuild child process is run in esbuild's API for Deno. Previously it used `Deno.run` but that API is being removed in favor of `Deno.Command`. As part of this change, esbuild is now calling the new `unref` function on esbuild's long-lived child process, which is supposed to allow Deno to exit when your code has finished running even though the child process is still around (previously you had to explicitly call esbuild's `stop()` function to terminate the child process for Deno to be able to exit).
+
+    However, this introduced a problem for Deno's testing API which now fails some tests that use esbuild with `error: Promise resolution is still pending but the event loop has already resolved`. It's unclear to me why this is happening. The call to `unref` was recommended by someone on the Deno core team, and calling Node's equivalent `unref` API has been working fine for esbuild in Node for a long time. It could be that I'm using it incorrectly, or that there's some reference counting and/or garbage collection bug in Deno's internals, or that Deno's `unref` just works differently than Node's `unref`. In any case, it's not good for Deno tests that use esbuild to be failing.
+
+    In this release, I am removing the call to `unref` to fix this issue. This means that you will now have to call esbuild's `stop()` function to allow Deno to exit, just like you did before esbuild version 0.20.0 when this regression was introduced.
+
+    Note: This regression wasn't caught earlier because Deno doesn't seem to fail tests that have outstanding `setTimeout` calls, which esbuild's test harness was using to enforce a maximum test runtime. Adding a `setTimeout` was allowing esbuild's Deno tests to succeed. So this regression doesn't necessarily apply to all people using tests in Deno.
+
 ## 0.20.1
 
 * Fix a bug with the CSS nesting transform ([#3648](https://github.com/evanw/esbuild/issues/3648))
diff -pruN 0.20.1-1/Makefile 0.20.2-1/Makefile
--- 0.20.1-1/Makefile	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/Makefile	2024-03-14 19:49:11.000000000 +0000
@@ -107,11 +107,14 @@ test-old-ts: platform-neutral | require/
 node-unref-tests: | scripts/node_modules
 	node scripts/node-unref-tests.js
 
-lib-typecheck: lib-typecheck-node lib-typecheck-deno
+lib-typecheck: lib-typecheck-node lib-typecheck-node-nolib lib-typecheck-deno
 
 lib-typecheck-node: | lib/node_modules
 	cd lib && node_modules/.bin/tsc -noEmit -p tsconfig.json
 
+lib-typecheck-node-nolib: | lib/node_modules
+	cd lib && node_modules/.bin/tsc -noEmit -p tsconfig-nolib.json
+
 lib-typecheck-deno: lib/deno/lib.deno.d.ts | lib/node_modules
 	cd lib && node_modules/.bin/tsc -noEmit -p tsconfig-deno.json
 
diff -pruN 0.20.1-1/cmd/esbuild/version.go 0.20.2-1/cmd/esbuild/version.go
--- 0.20.1-1/cmd/esbuild/version.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/cmd/esbuild/version.go	2024-03-14 19:49:11.000000000 +0000
@@ -1,3 +1,3 @@
 package main
 
-const esbuildVersion = "0.20.1"
+const esbuildVersion = "0.20.2"
diff -pruN 0.20.1-1/debian/changelog 0.20.2-1/debian/changelog
--- 0.20.1-1/debian/changelog	2024-02-21 16:02:31.000000000 +0000
+++ 0.20.2-1/debian/changelog	2024-03-25 02:28:41.000000000 +0000
@@ -1,3 +1,10 @@
+golang-github-evanw-esbuild (0.20.2-1) unstable; urgency=medium
+
+  * New upstream version 0.20.2
+  * Update esbuild(1) man page to 0.20.2
+
+ -- Anthony Fok <foka@debian.org>  Sun, 24 Mar 2024 20:28:41 -0600
+
 golang-github-evanw-esbuild (0.20.1-1) unstable; urgency=medium
 
   * New upstream version 0.20.1
diff -pruN 0.20.1-1/debian/esbuild.1 0.20.2-1/debian/esbuild.1
--- 0.20.1-1/debian/esbuild.1	2024-02-21 16:00:06.000000000 +0000
+++ 0.20.2-1/debian/esbuild.1	2024-03-25 02:28:24.000000000 +0000
@@ -4,7 +4,7 @@
 .\" First parameter, NAME, should be all caps
 .\" Second parameter, SECTION, should be 1-8, maybe w/ subsection
 .\" other parameters are allowed: see man(7), man(1)
-.TH ESBUILD "1" "2024-02-18" "esbuild 0.20.1" "esbuild Manual"
+.TH ESBUILD "1" "2024-03-14" "esbuild 0.20.2" "esbuild Manual"
 .\" Please adjust this date whenever revising the manpage.
 .\"
 .\" Some roff macros, for reference:
@@ -264,7 +264,7 @@ Force tree shaking on or off (false | tr
 Use this tsconfig.json file instead of other ones
 .TP
 .B \-\-version
-Print the current version (0.20.1) and exit
+Print the current version (0.20.2) and exit
 .SH EXAMPLES
 .TP 4
 .B esbuild \-\-bundle entry_point.js \-\-outdir=dist \-\-minify \-\-sourcemap
diff -pruN 0.20.1-1/internal/bundler/bundler.go 0.20.2-1/internal/bundler/bundler.go
--- 0.20.1-1/internal/bundler/bundler.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler/bundler.go	2024-03-14 19:49:11.000000000 +0000
@@ -143,7 +143,6 @@ func parseFile(args parseArgs) {
 	} else {
 		result, ok := runOnLoadPlugins(
 			args.options.Plugins,
-			args.res,
 			args.fs,
 			&args.caches.FSCache,
 			args.log,
@@ -245,7 +244,9 @@ func parseFile(args parseArgs) {
 		result.ok = true
 
 	case config.LoaderJSON, config.LoaderWithTypeJSON:
-		expr, ok := args.caches.JSONCache.Parse(args.log, source, js_parser.JSONOptions{})
+		expr, ok := args.caches.JSONCache.Parse(args.log, source, js_parser.JSONOptions{
+			UnsupportedJSFeatures: args.options.UnsupportedJSFeatures,
+		})
 		ast := js_parser.LazyExportAST(args.log, source, js_parser.OptionsFromConfig(&args.options), expr, "")
 		if loader == config.LoaderWithTypeJSON {
 			// The exports kind defaults to "none", in which case the linker picks
@@ -540,7 +541,7 @@ func parseFile(args parseArgs) {
 				tracker := logger.MakeLineColumnTracker(&source)
 
 				if path, contents := extractSourceMapFromComment(args.log, args.fs, &args.caches.FSCache,
-					args.res, &source, &tracker, sourceMapComment, absResolveDir); contents != nil {
+					&source, &tracker, sourceMapComment, absResolveDir); contents != nil {
 					prettyPath := resolver.PrettyPath(args.fs, path)
 					log := logger.NewDeferLog(logger.DeferLogNoVerboseOrDebug, args.log.Overrides)
 
@@ -743,7 +744,6 @@ func extractSourceMapFromComment(
 	log logger.Log,
 	fs fs.FS,
 	fsCache *cache.FSCache,
-	res *resolver.Resolver,
 	source *logger.Source,
 	tracker *logger.LineColumnTracker,
 	comment logger.Span,
@@ -982,7 +982,6 @@ type loaderPluginResult struct {
 
 func runOnLoadPlugins(
 	plugins []config.Plugin,
-	res *resolver.Resolver,
 	fs fs.FS,
 	fsCache *cache.FSCache,
 	log logger.Log,
diff -pruN 0.20.1-1/internal/bundler_tests/bundler_loader_test.go 0.20.2-1/internal/bundler_tests/bundler_loader_test.go
--- 0.20.1-1/internal/bundler_tests/bundler_loader_test.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler_tests/bundler_loader_test.go	2024-03-14 19:49:11.000000000 +0000
@@ -1632,3 +1632,46 @@ func TestLoaderBundleWithTypeJSONOnlyDef
 `,
 	})
 }
+
+func TestLoaderJSONPrototype(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import data from "./data.json"
+				console.log(data)
+			`,
+			"/data.json": `{
+				"": "The property below should be converted to a computed property:",
+				"__proto__": { "foo": "bar" }
+			}`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MinifySyntax:  true,
+		},
+	})
+}
+
+func TestLoaderJSONPrototypeES5(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import data from "./data.json"
+				console.log(data)
+			`,
+			"/data.json": `{
+				"": "The property below should NOT be converted to a computed property for ES5:",
+				"__proto__": { "foo": "bar" }
+			}`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			AbsOutputFile:         "/out.js",
+			MinifySyntax:          true,
+			UnsupportedJSFeatures: es(5),
+		},
+	})
+}
diff -pruN 0.20.1-1/internal/bundler_tests/bundler_ts_test.go 0.20.2-1/internal/bundler_tests/bundler_ts_test.go
--- 0.20.1-1/internal/bundler_tests/bundler_ts_test.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler_tests/bundler_ts_test.go	2024-03-14 19:49:11.000000000 +0000
@@ -1054,6 +1054,7 @@ func TestTSExperimentalDecorators(t *tes
 					@x @y mDef = 1
 					@x @y method(@x0 @y0 arg0, @x1 @y1 arg1) { return new Foo }
 					@x @y declare mDecl
+					@x @y abstract mAbst
 					constructor(@x0 @y0 arg0, @x1 @y1 arg1) {}
 
 					@x @y static sUndef
@@ -1070,6 +1071,7 @@ func TestTSExperimentalDecorators(t *tes
 					@x @y [mDef()] = 1
 					@x @y [method()](@x0 @y0 arg0, @x1 @y1 arg1) { return new Foo }
 					@x @y declare [mDecl()]
+					@x @y abstract [mAbst()]
 
 					// Side effect order must be preserved even for fields without decorators
 					[xUndef()]
diff -pruN 0.20.1-1/internal/bundler_tests/bundler_tsconfig_test.go 0.20.2-1/internal/bundler_tests/bundler_tsconfig_test.go
--- 0.20.1-1/internal/bundler_tests/bundler_tsconfig_test.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler_tests/bundler_tsconfig_test.go	2024-03-14 19:49:11.000000000 +0000
@@ -2581,3 +2581,76 @@ func TestTsconfigJsonAsteriskNameCollisi
 		},
 	})
 }
+
+// https://github.com/evanw/esbuild/issues/3698
+func TestTsconfigPackageJsonExportsYarnPnP(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/packages/app/index.tsx": `
+				console.log(<div/>)
+			`,
+			"/Users/user/project/packages/app/tsconfig.json": `
+				{
+					"extends": "tsconfigs/config"
+				}
+			`,
+			"/Users/user/project/packages/tsconfigs/package.json": `
+				{
+					"exports": {
+						"./config": "./configs/tsconfig.json"
+					}
+				}
+			`,
+			"/Users/user/project/packages/tsconfigs/configs/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"jsxFactory": "success"
+					}
+				}
+			`,
+			"/Users/user/project/.pnp.data.json": `
+				{
+					"packageRegistryData": [
+						[
+							"app",
+							[
+								[
+									"workspace:packages/app",
+									{
+										"packageLocation": "./packages/app/",
+										"packageDependencies": [
+											[
+												"tsconfigs",
+												"workspace:packages/tsconfigs"
+											]
+										],
+										"linkType": "SOFT"
+									}
+								]
+							]
+						],
+						[
+							"tsconfigs",
+							[
+								[
+									"workspace:packages/tsconfigs",
+									{
+										"packageLocation": "./packages/tsconfigs/",
+										"packageDependencies": [],
+										"linkType": "SOFT"
+									}
+								]
+							]
+						]
+					]
+				}
+			`,
+		},
+		entryPaths:    []string{"/Users/user/project/packages/app/index.tsx"},
+		absWorkingDir: "/Users/user/project",
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
diff -pruN 0.20.1-1/internal/bundler_tests/snapshots/snapshots_loader.txt 0.20.2-1/internal/bundler_tests/snapshots/snapshots_loader.txt
--- 0.20.1-1/internal/bundler_tests/snapshots/snapshots_loader.txt	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler_tests/snapshots/snapshots_loader.txt	2024-03-14 19:49:11.000000000 +0000
@@ -900,6 +900,30 @@ TestLoaderJSONNoBundleIIFE
 })();
 
 ================================================================================
+TestLoaderJSONPrototype
+---------- /out.js ----------
+// data.json
+var data_default = {
+  "": "The property below should be converted to a computed property:",
+  ["__proto__"]: { foo: "bar" }
+};
+
+// entry.js
+console.log(data_default);
+
+================================================================================
+TestLoaderJSONPrototypeES5
+---------- /out.js ----------
+// data.json
+var data_default = {
+  "": "The property below should NOT be converted to a computed property for ES5:",
+  __proto__: { foo: "bar" }
+};
+
+// entry.js
+console.log(data_default);
+
+================================================================================
 TestLoaderJSONSharedWithMultipleEntriesIssue413
 ---------- /out/a.js ----------
 // data.json
diff -pruN 0.20.1-1/internal/bundler_tests/snapshots/snapshots_ts.txt 0.20.2-1/internal/bundler_tests/snapshots/snapshots_ts.txt
--- 0.20.1-1/internal/bundler_tests/snapshots/snapshots_ts.txt	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler_tests/snapshots/snapshots_ts.txt	2024-03-14 19:49:11.000000000 +0000
@@ -720,6 +720,10 @@ __decorateClass([
 __decorateClass([
   x,
   y
+], Foo.prototype, "mAbst", 2);
+__decorateClass([
+  x,
+  y
 ], Foo, "sUndef", 2);
 __decorateClass([
   x,
@@ -747,24 +751,24 @@ Foo = __decorateClass([
 ], Foo);
 
 // all_computed.ts
-var _a, _b, _c, _d, _e, _f, _g, _h, _i, _j;
+var _a, _b, _c, _d, _e, _f, _g, _h, _i, _j, _k;
 var Foo2 = class {
   constructor() {
     this[_b] = 1;
-    this[_e] = 2;
+    this[_f] = 2;
   }
   static {
-    _j = mDecl();
+    _k = mDecl();
   }
   [(_a = mUndef(), _b = mDef(), _c = method())](arg0, arg1) {
     return new Foo2();
   }
-  static [(_d = mDecl(), xUndef(), _e = xDef(), yUndef(), _f = yDef(), _g = sUndef(), _h = sDef(), _i = sMethod())](arg0, arg1) {
+  static [(_d = mDecl(), _e = mAbst(), xUndef(), _f = xDef(), yUndef(), _g = yDef(), _h = sUndef(), _i = sDef(), _j = sMethod())](arg0, arg1) {
     return new Foo2();
   }
 };
-Foo2[_f] = 3;
-Foo2[_h] = new Foo2();
+Foo2[_g] = 3;
+Foo2[_i] = new Foo2();
 __decorateClass([
   x,
   y
@@ -788,23 +792,27 @@ __decorateClass([
 __decorateClass([
   x,
   y
-], Foo2, _g, 2);
+], Foo2.prototype, _e, 2);
 __decorateClass([
   x,
   y
 ], Foo2, _h, 2);
 __decorateClass([
   x,
+  y
+], Foo2, _i, 2);
+__decorateClass([
+  x,
   y,
   __decorateParam(0, x0),
   __decorateParam(0, y0),
   __decorateParam(1, x1),
   __decorateParam(1, y1)
-], Foo2, _i, 1);
+], Foo2, _j, 1);
 __decorateClass([
   x,
   y
-], Foo2, _j, 2);
+], Foo2, _k, 2);
 Foo2 = __decorateClass([
   x?.[_ + "y"](),
   new y?.[_ + "x"]()
diff -pruN 0.20.1-1/internal/bundler_tests/snapshots/snapshots_tsconfig.txt 0.20.2-1/internal/bundler_tests/snapshots/snapshots_tsconfig.txt
--- 0.20.1-1/internal/bundler_tests/snapshots/snapshots_tsconfig.txt	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/bundler_tests/snapshots/snapshots_tsconfig.txt	2024-03-14 19:49:11.000000000 +0000
@@ -397,6 +397,12 @@ var both_default = /* @__PURE__ */ R.c(R
 console.log(factory_default, fragment_default, both_default);
 
 ================================================================================
+TestTsconfigPackageJsonExportsYarnPnP
+---------- /Users/user/project/out.js ----------
+// packages/app/index.tsx
+console.log(/* @__PURE__ */ success("div", null));
+
+================================================================================
 TestTsconfigPaths
 ---------- /Users/user/project/out.js ----------
 // Users/user/project/baseurl_dot/test0-success.ts
diff -pruN 0.20.1-1/internal/css_parser/css_nesting.go 0.20.2-1/internal/css_parser/css_nesting.go
--- 0.20.1-1/internal/css_parser/css_nesting.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/css_parser/css_nesting.go	2024-03-14 19:49:11.000000000 +0000
@@ -190,7 +190,7 @@ func (p *parser) lowerNestingInRuleWithC
 			r.Selectors = []css_ast.ComplexSelector{merged}
 		}
 
-		// Pass 2: Substitue "&" for the parent selector
+		// Pass 2: Substitute "&" for the parent selector
 		if !p.options.unsupportedCSSFeatures.Has(compat.IsPseudoClass) || len(context.parentSelectors) <= 1 {
 			// If we can use ":is", or we don't have to because there's only one
 			// parent selector, or we are using ":is()" to match zero parent selectors
diff -pruN 0.20.1-1/internal/js_ast/js_ast.go 0.20.2-1/internal/js_ast/js_ast.go
--- 0.20.1-1/internal/js_ast/js_ast.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_ast/js_ast.go	2024-03-14 19:49:11.000000000 +0000
@@ -255,7 +255,7 @@ const (
 	PropertySet
 	PropertyAutoAccessor
 	PropertySpread
-	PropertyDeclare
+	PropertyDeclareOrAbstract
 	PropertyClassStaticBlock
 )
 
diff -pruN 0.20.1-1/internal/js_parser/js_parser.go 0.20.2-1/internal/js_parser/js_parser.go
--- 0.20.1-1/internal/js_parser/js_parser.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_parser/js_parser.go	2024-03-14 19:49:11.000000000 +0000
@@ -2204,7 +2204,7 @@ func (p *parser) parseProperty(startLoc
 							//   https://github.com/evanw/esbuild/issues/1675
 							//   https://github.com/microsoft/TypeScript/issues/46345
 							//
-							prop.Kind = js_ast.PropertyDeclare
+							prop.Kind = js_ast.PropertyDeclareOrAbstract
 							return prop, true
 						}
 
@@ -2216,7 +2216,33 @@ func (p *parser) parseProperty(startLoc
 					if !p.lexer.HasNewlineBefore && opts.isClass && p.options.ts.Parse && !opts.isTSAbstract && raw == name.String {
 						opts.isTSAbstract = true
 						scopeIndex := len(p.scopesInOrder)
-						p.parseProperty(startLoc, kind, opts, nil)
+
+						if prop, ok := p.parseProperty(startLoc, kind, opts, nil); ok &&
+							prop.Kind == js_ast.PropertyNormal && prop.ValueOrNil.Data == nil &&
+							(p.options.ts.Config.ExperimentalDecorators == config.True && len(opts.decorators) > 0) {
+							// If this is a well-formed class field with the "abstract" keyword,
+							// only keep the declaration to preserve its side-effects when
+							// there are TypeScript experimental decorators present:
+							//
+							//   abstract class Foo {
+							//     // Remove this
+							//     abstract [(console.log('side effect 1'), 'foo')]
+							//
+							//     // Keep this
+							//     @decorator(console.log('side effect 2')) abstract bar
+							//   }
+							//
+							// This behavior is valid with TypeScript experimental decorators.
+							// TypeScript does not allow this with JavaScript decorators.
+							//
+							// References:
+							//
+							//   https://github.com/evanw/esbuild/issues/3684
+							//
+							prop.Kind = js_ast.PropertyDeclareOrAbstract
+							return prop, true
+						}
+
 						p.discardScopesUpTo(scopeIndex)
 						return js_ast.Property{}, false
 					}
@@ -6687,53 +6713,94 @@ func (p *parser) parseDecorator() js_ast
 
 	memberExpr := js_ast.Expr{Loc: nameRange.Loc, Data: &js_ast.EIdentifier{Ref: p.storeNameInRef(name)}}
 
-	// "@x<y>() class{}"
-	if p.options.ts.Parse {
-		p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{})
-	}
+	// Custom error reporting for error recovery
+	var syntaxError logger.MsgData
+	wrapRange := nameRange
 
-	for p.lexer.Token == js_lexer.TDot {
-		p.lexer.Next()
+loop:
+	for {
+		switch p.lexer.Token {
+		case js_lexer.TExclamation:
+			// Skip over TypeScript non-null assertions
+			if p.lexer.HasNewlineBefore {
+				break loop
+			}
+			if !p.options.ts.Parse {
+				p.lexer.Unexpected()
+			}
+			wrapRange.Len = p.lexer.Range().End() - wrapRange.Loc.Start
+			p.lexer.Next()
 
-		if p.lexer.Token == js_lexer.TPrivateIdentifier {
-			name := p.lexer.Identifier
-			memberExpr.Data = &js_ast.EIndex{
-				Target: memberExpr,
-				Index:  js_ast.Expr{Loc: p.lexer.Loc(), Data: &js_ast.EPrivateIdentifier{Ref: p.storeNameInRef(name)}},
+		case js_lexer.TDot, js_lexer.TQuestionDot:
+			// The grammar for "DecoratorMemberExpression" currently forbids "?."
+			if p.lexer.Token == js_lexer.TQuestionDot && syntaxError.Location == nil {
+				syntaxError = p.tracker.MsgData(p.lexer.Range(), "JavaScript decorator syntax does not allow \"?.\" here")
 			}
-			p.reportPrivateNameUsage(name.String)
+
 			p.lexer.Next()
-		} else {
-			memberExpr.Data = &js_ast.EDot{
-				Target:  memberExpr,
-				Name:    p.lexer.Identifier.String,
-				NameLoc: p.lexer.Loc(),
+			wrapRange.Len = p.lexer.Range().End() - wrapRange.Loc.Start
+
+			if p.lexer.Token == js_lexer.TPrivateIdentifier {
+				name := p.lexer.Identifier
+				memberExpr.Data = &js_ast.EIndex{
+					Target: memberExpr,
+					Index:  js_ast.Expr{Loc: p.lexer.Loc(), Data: &js_ast.EPrivateIdentifier{Ref: p.storeNameInRef(name)}},
+				}
+				p.reportPrivateNameUsage(name.String)
+				p.lexer.Next()
+			} else {
+				memberExpr.Data = &js_ast.EDot{
+					Target:  memberExpr,
+					Name:    p.lexer.Identifier.String,
+					NameLoc: p.lexer.Loc(),
+				}
+				p.lexer.Expect(js_lexer.TIdentifier)
 			}
-			p.lexer.Expect(js_lexer.TIdentifier)
-		}
 
-		// "@x.y<z>() class{}"
-		if p.options.ts.Parse {
-			p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{})
-		}
-	}
+		case js_lexer.TOpenParen:
+			args, closeParenLoc, isMultiLine := p.parseCallArgs()
+			memberExpr.Data = &js_ast.ECall{
+				Target:        memberExpr,
+				Args:          args,
+				CloseParenLoc: closeParenLoc,
+				IsMultiLine:   isMultiLine,
+				Kind:          js_ast.TargetWasOriginallyPropertyAccess,
+			}
+			wrapRange.Len = closeParenLoc.Start + 1 - wrapRange.Loc.Start
 
-	// The grammar for "DecoratorMemberExpression" currently forbids "?."
-	if p.lexer.Token == js_lexer.TQuestionDot {
-		p.lexer.Expect(js_lexer.TDot)
-	}
+			// The grammar for "DecoratorCallExpression" currently forbids anything after it
+			if p.lexer.Token == js_lexer.TDot {
+				if syntaxError.Location == nil {
+					syntaxError = p.tracker.MsgData(p.lexer.Range(), "JavaScript decorator syntax does not allow \".\" after a call expression")
+				}
+				continue
+			}
+			break loop
 
-	if p.lexer.Token == js_lexer.TOpenParen {
-		args, closeParenLoc, isMultiLine := p.parseCallArgs()
-		memberExpr.Data = &js_ast.ECall{
-			Target:        memberExpr,
-			Args:          args,
-			CloseParenLoc: closeParenLoc,
-			IsMultiLine:   isMultiLine,
-			Kind:          js_ast.TargetWasOriginallyPropertyAccess,
+		default:
+			// "@x<y>"
+			// "@x.y<z>"
+			if !p.skipTypeScriptTypeArguments(skipTypeScriptTypeArgumentsOpts{}) {
+				break loop
+			}
 		}
 	}
 
+	// Suggest that non-decorator expressions be wrapped in parentheses
+	if syntaxError.Location != nil {
+		var notes []logger.MsgData
+		if text := p.source.TextForRange(wrapRange); !strings.ContainsRune(text, '\n') {
+			note := p.tracker.MsgData(wrapRange, "Wrap this decorator in parentheses to allow arbitrary expressions:")
+			note.Location.Suggestion = fmt.Sprintf("(%s)", text)
+			notes = []logger.MsgData{note}
+		}
+		p.log.AddMsg(logger.Msg{
+			Kind:  logger.Error,
+			Data:  syntaxError,
+			Notes: notes,
+		})
+	}
+
 	return memberExpr
 }
 
@@ -8355,6 +8422,9 @@ func shouldKeepStmtInDeadControlFlow(stm
 		for _, decl := range s.Decls {
 			identifiers = findIdentifiers(decl.Binding, identifiers)
 		}
+		if len(identifiers) == 0 {
+			return false
+		}
 		s.Decls = identifiers
 		return true
 
@@ -8557,7 +8627,7 @@ func (p *parser) visitStmts(stmts []js_a
 		for _, stmt := range before {
 			s, ok := stmt.Data.(*js_ast.SFunction)
 			if !ok {
-				// We may get non-function statements here in certain scenarious such as when "KeepNames" is enabled
+				// We may get non-function statements here in certain scenarios such as when "KeepNames" is enabled
 				nonFnStmts = append(nonFnStmts, stmt)
 				continue
 			}
@@ -10701,6 +10771,14 @@ func (p *parser) visitAndAppendStmt(stmt
 			}
 		}
 
+		// Unwrap switch statements in dead code
+		if p.options.minifySyntax && p.isControlFlowDead {
+			for _, c := range s.Cases {
+				stmts = append(stmts, c.Body...)
+			}
+			return stmts
+		}
+
 		// "using" declarations inside switch statements must be special-cased
 		if lowered := p.maybeLowerUsingDeclarationsInSwitch(stmt.Loc, s); lowered != nil {
 			return append(stmts, lowered...)
diff -pruN 0.20.1-1/internal/js_parser/js_parser_lower_class.go 0.20.2-1/internal/js_parser/js_parser_lower_class.go
--- 0.20.1-1/internal/js_parser/js_parser_lower_class.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_parser/js_parser_lower_class.go	2024-03-14 19:49:11.000000000 +0000
@@ -1062,10 +1062,10 @@ func (p *parser) lowerClass(stmt js_ast.
 			}
 		}
 
-		// If the field uses the TypeScript "declare" keyword, just omit it entirely.
-		// However, we must still keep any side-effects in the computed value and/or
-		// in the decorators.
-		if prop.Kind == js_ast.PropertyDeclare && prop.ValueOrNil.Data == nil {
+		// If the field uses the TypeScript "declare" or "abstract" keyword, just
+		// omit it entirely. However, we must still keep any side-effects in the
+		// computed value and/or in the decorators.
+		if prop.Kind == js_ast.PropertyDeclareOrAbstract && prop.ValueOrNil.Data == nil {
 			mustLowerField = true
 			shouldOmitFieldInitializer = true
 		}
diff -pruN 0.20.1-1/internal/js_parser/js_parser_test.go 0.20.2-1/internal/js_parser/js_parser_test.go
--- 0.20.1-1/internal/js_parser/js_parser_test.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_parser/js_parser_test.go	2024-03-14 19:49:11.000000000 +0000
@@ -2028,15 +2028,19 @@ func TestDecorators(t *testing.T) {
 	expectPrinted(t, "class Foo { #x = @y.#x.y.#x class {} }", "class Foo {\n  #x = @y.#x.y.#x class {\n  };\n}\n")
 	expectParseError(t, "@123 class Foo {}", "<stdin>: ERROR: Expected identifier but found \"123\"\n")
 	expectParseError(t, "@x[y] class Foo {}", "<stdin>: ERROR: Expected \";\" but found \"class\"\n")
-	expectParseError(t, "@x?.() class Foo {}", "<stdin>: ERROR: Expected \".\" but found \"?.\"\n")
-	expectParseError(t, "@x?.y() class Foo {}", "<stdin>: ERROR: Expected \".\" but found \"?.\"\n")
-	expectParseError(t, "@x?.[y]() class Foo {}", "<stdin>: ERROR: Expected \".\" but found \"?.\"\n")
+	expectParseError(t, "@x?.() class Foo {}", "<stdin>: ERROR: Expected identifier but found \"(\"\n")
+	expectParseError(t, "@x?.y() class Foo {}",
+		"<stdin>: ERROR: JavaScript decorator syntax does not allow \"?.\" here\n"+
+			"<stdin>: NOTE: Wrap this decorator in parentheses to allow arbitrary expressions:\n")
+	expectParseError(t, "@x?.[y]() class Foo {}", "<stdin>: ERROR: Expected identifier but found \"[\"\n")
 	expectParseError(t, "@new Function() class Foo {}", "<stdin>: ERROR: Expected identifier but found \"new\"\n")
 	expectParseError(t, "@() => {} class Foo {}", "<stdin>: ERROR: Unexpected \")\"\n")
 	expectParseError(t, "x = @y function() {}", "<stdin>: ERROR: Expected \"class\" but found \"function\"\n")
 
 	// See: https://github.com/microsoft/TypeScript/issues/55336
-	expectParseError(t, "@x().y() class Foo {}", "<stdin>: ERROR: Unexpected \".\"\n")
+	expectParseError(t, "@x().y() class Foo {}",
+		"<stdin>: ERROR: JavaScript decorator syntax does not allow \".\" after a call expression\n"+
+			"<stdin>: NOTE: Wrap this decorator in parentheses to allow arbitrary expressions:\n")
 
 	errorText := "<stdin>: ERROR: Transforming JavaScript decorators to the configured target environment is not supported yet\n"
 	expectParseErrorWithUnsupportedFeatures(t, compat.Decorators, "@dec class Foo {}", errorText)
@@ -2064,6 +2068,9 @@ func TestDecorators(t *testing.T) {
 	expectParseError(t, "@x export @y class Foo {}", "<stdin>: ERROR: Decorators are not valid here\n")
 	expectParseError(t, "@x export default abstract", "<stdin>: ERROR: Decorators are not valid here\n")
 	expectParseError(t, "@x export @y default class {}", "<stdin>: ERROR: Decorators are not valid here\n<stdin>: ERROR: Unexpected \"default\"\n")
+
+	// Disallow TypeScript syntax in JavaScript
+	expectParseError(t, "@x!.y!.z class Foo {}", "<stdin>: ERROR: Unexpected \"!\"\n")
 }
 
 func TestGenerator(t *testing.T) {
@@ -5232,6 +5239,7 @@ func TestTrimCodeInDeadControlFlow(t *te
 	expectPrintedMangle(t, "b: while (1) if (0) a(); else { continue b }", "b:\n  for (; ; )\n    continue b;\n")
 	expectPrintedMangle(t, "if (1) a(); else { class b {} }", "a();\n")
 	expectPrintedMangle(t, "if (1) a(); else { debugger }", "a();\n")
+	expectPrintedMangle(t, "if (1) a(); else { switch (1) { case 1: b() } }", "a();\n")
 
 	expectPrintedMangle(t, "if (0) {let a = 1} else a()", "a();\n")
 	expectPrintedMangle(t, "if (1) {let a = 1} else a()", "{\n  let a = 1;\n}\n")
@@ -5241,9 +5249,12 @@ func TestTrimCodeInDeadControlFlow(t *te
 	expectPrintedMangle(t, "if (1) a(); else { var a = b }", "if (1)\n  a();\nelse\n  var a;\n")
 	expectPrintedMangle(t, "if (1) a(); else { var [a] = b }", "if (1)\n  a();\nelse\n  var a;\n")
 	expectPrintedMangle(t, "if (1) a(); else { var {x: a} = b }", "if (1)\n  a();\nelse\n  var a;\n")
+	expectPrintedMangle(t, "if (1) a(); else { var [] = b }", "a();\n")
+	expectPrintedMangle(t, "if (1) a(); else { var {} = b }", "a();\n")
 	expectPrintedMangle(t, "if (1) a(); else { function a() {} }", "if (1)\n  a();\nelse\n  var a;\n")
 	expectPrintedMangle(t, "if (1) a(); else { for(;;){var a} }", "if (1)\n  a();\nelse\n  for (; ; )\n    var a;\n")
 	expectPrintedMangle(t, "if (1) { a(); b() } else { var a; var b; }", "if (1)\n  a(), b();\nelse\n  var a, b;\n")
+	expectPrintedMangle(t, "if (1) a(); else { switch (1) { case 1: case 2: var a } }", "if (1)\n  a();\nelse\n  var a;\n")
 }
 
 func TestPreservedComments(t *testing.T) {
diff -pruN 0.20.1-1/internal/js_parser/json_parser.go 0.20.2-1/internal/js_parser/json_parser.go
--- 0.20.1-1/internal/js_parser/json_parser.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_parser/json_parser.go	2024-03-14 19:49:11.000000000 +0000
@@ -3,6 +3,7 @@ package js_parser
 import (
 	"fmt"
 
+	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/helpers"
 	"github.com/evanw/esbuild/internal/js_ast"
 	"github.com/evanw/esbuild/internal/js_lexer"
@@ -142,6 +143,14 @@ func (p *jsonParser) parseExpr() js_ast.
 				Key:        key,
 				ValueOrNil: value,
 			}
+
+			// The key "__proto__" must not be a string literal in JavaScript because
+			// that actually modifies the prototype of the object. This can be
+			// avoided by using a computed property key instead of a string literal.
+			if helpers.UTF16EqualsString(keyString, "__proto__") && !p.options.UnsupportedJSFeatures.Has(compat.ObjectExtensions) {
+				property.Flags |= js_ast.PropertyIsComputed
+			}
+
 			properties = append(properties, property)
 		}
 
@@ -163,8 +172,9 @@ func (p *jsonParser) parseExpr() js_ast.
 }
 
 type JSONOptions struct {
-	Flavor      js_lexer.JSONFlavor
-	ErrorSuffix string
+	UnsupportedJSFeatures compat.JSFeature
+	Flavor                js_lexer.JSONFlavor
+	ErrorSuffix           string
 }
 
 func ParseJSON(log logger.Log, source logger.Source, options JSONOptions) (result js_ast.Expr, ok bool) {
diff -pruN 0.20.1-1/internal/js_parser/ts_parser.go 0.20.2-1/internal/js_parser/ts_parser.go
--- 0.20.1-1/internal/js_parser/ts_parser.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_parser/ts_parser.go	2024-03-14 19:49:11.000000000 +0000
@@ -1930,19 +1930,17 @@ func (p *parser) generateClosureForTypeS
 
 	// Generate the body of the closure, including a return statement at the end
 	stmtsInsideClosure := []js_ast.Stmt{}
-	if len(exprsInsideClosure) > 0 {
-		argExpr := js_ast.Expr{Loc: nameLoc, Data: &js_ast.EIdentifier{Ref: argRef}}
-		if p.options.minifySyntax {
-			// "a; b; return c;" => "return a, b, c;"
-			joined := js_ast.JoinAllWithComma(exprsInsideClosure)
-			joined = js_ast.JoinWithComma(joined, argExpr)
-			stmtsInsideClosure = append(stmtsInsideClosure, js_ast.Stmt{Loc: joined.Loc, Data: &js_ast.SReturn{ValueOrNil: joined}})
-		} else {
-			for _, expr := range exprsInsideClosure {
-				stmtsInsideClosure = append(stmtsInsideClosure, js_ast.Stmt{Loc: expr.Loc, Data: &js_ast.SExpr{Value: expr}})
-			}
-			stmtsInsideClosure = append(stmtsInsideClosure, js_ast.Stmt{Loc: argExpr.Loc, Data: &js_ast.SReturn{ValueOrNil: argExpr}})
+	argExpr := js_ast.Expr{Loc: nameLoc, Data: &js_ast.EIdentifier{Ref: argRef}}
+	if p.options.minifySyntax {
+		// "a; b; return c;" => "return a, b, c;"
+		joined := js_ast.JoinAllWithComma(exprsInsideClosure)
+		joined = js_ast.JoinWithComma(joined, argExpr)
+		stmtsInsideClosure = append(stmtsInsideClosure, js_ast.Stmt{Loc: joined.Loc, Data: &js_ast.SReturn{ValueOrNil: joined}})
+	} else {
+		for _, expr := range exprsInsideClosure {
+			stmtsInsideClosure = append(stmtsInsideClosure, js_ast.Stmt{Loc: expr.Loc, Data: &js_ast.SExpr{Value: expr}})
 		}
+		stmtsInsideClosure = append(stmtsInsideClosure, js_ast.Stmt{Loc: argExpr.Loc, Data: &js_ast.SReturn{ValueOrNil: argExpr}})
 	}
 
 	// Try to use an arrow function if possible for compactness
diff -pruN 0.20.1-1/internal/js_parser/ts_parser_test.go 0.20.2-1/internal/js_parser/ts_parser_test.go
--- 0.20.1-1/internal/js_parser/ts_parser_test.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_parser/ts_parser_test.go	2024-03-14 19:49:11.000000000 +0000
@@ -1804,7 +1804,7 @@ func TestTSDeclare(t *testing.T) {
 	expectPrintedTS(t, "declare\nconst foo = 0", "declare;\nconst foo = 0;\n")
 	expectPrintedTS(t, "declare\nfunction foo() {}", "declare;\nfunction foo() {\n}\n")
 	expectPrintedTS(t, "declare\nclass Foo {}", "declare;\nclass Foo {\n}\n")
-	expectPrintedTS(t, "declare\nenum Foo {}", "declare;\nvar Foo = /* @__PURE__ */ ((Foo) => {\n})(Foo || {});\n")
+	expectPrintedTS(t, "declare\nenum Foo {}", "declare;\nvar Foo = /* @__PURE__ */ ((Foo) => {\n  return Foo;\n})(Foo || {});\n")
 	expectPrintedTS(t, "class Foo { declare \n foo }", "class Foo {\n  declare;\n  foo;\n}\n")
 
 	expectPrintedTS(t, "declare;", "declare;\n")
@@ -2051,9 +2051,15 @@ func TestTSExperimentalDecorator(t *test
 	expectParseErrorExperimentalDecoratorTS(t, "@x export default abstract", "<stdin>: ERROR: Decorators are not valid here\n")
 	expectParseErrorExperimentalDecoratorTS(t, "@x export @y default class {}", "<stdin>: ERROR: Decorators are not valid here\n<stdin>: ERROR: Unexpected \"default\"\n")
 
-	// TypeScript experimental decorators are actually allowed on declared fields
+	// From the TypeScript team: "We do allow postfix ! because it's TypeScript only."
+	// https://github.com/microsoft/TypeScript/issues/57756
+	expectPrintedExperimentalDecoratorTS(t, "@x!.y!.z class Foo {}", "let Foo = class {\n};\nFoo = __decorateClass([\n  x.y.z\n], Foo);\n")
+
+	// TypeScript experimental decorators are actually allowed on declared and abstract fields
 	expectPrintedExperimentalDecoratorTS(t, "class Foo { @(() => {}) declare foo: any; @(() => {}) bar: any }",
 		"class Foo {\n  bar;\n}\n__decorateClass([\n  () => {\n  }\n], Foo.prototype, \"foo\", 2);\n__decorateClass([\n  () => {\n  }\n], Foo.prototype, \"bar\", 2);\n")
+	expectPrintedExperimentalDecoratorTS(t, "abstract class Foo { @(() => {}) abstract foo: any; @(() => {}) bar: any }",
+		"class Foo {\n  bar;\n}\n__decorateClass([\n  () => {\n  }\n], Foo.prototype, \"foo\", 2);\n__decorateClass([\n  () => {\n  }\n], Foo.prototype, \"bar\", 2);\n")
 }
 
 func TestTSDecorators(t *testing.T) {
@@ -2086,9 +2092,11 @@ func TestTSDecorators(t *testing.T) {
 	expectPrintedTS(t, "class Foo { #x = @y.#x.y.#x class {} }", "class Foo {\n  #x = @y.#x.y.#x class {\n  };\n}\n")
 	expectParseErrorTS(t, "@123 class Foo {}", "<stdin>: ERROR: Expected identifier but found \"123\"\n")
 	expectParseErrorTS(t, "@x[y] class Foo {}", "<stdin>: ERROR: Expected \";\" but found \"class\"\n")
-	expectParseErrorTS(t, "@x?.() class Foo {}", "<stdin>: ERROR: Expected \".\" but found \"?.\"\n")
-	expectParseErrorTS(t, "@x?.y() class Foo {}", "<stdin>: ERROR: Expected \".\" but found \"?.\"\n")
-	expectParseErrorTS(t, "@x?.[y]() class Foo {}", "<stdin>: ERROR: Expected \".\" but found \"?.\"\n")
+	expectParseErrorTS(t, "@x?.() class Foo {}", "<stdin>: ERROR: Expected identifier but found \"(\"\n")
+	expectParseErrorTS(t, "@x?.y() class Foo {}",
+		"<stdin>: ERROR: JavaScript decorator syntax does not allow \"?.\" here\n"+
+			"<stdin>: NOTE: Wrap this decorator in parentheses to allow arbitrary expressions:\n")
+	expectParseErrorTS(t, "@x?.[y]() class Foo {}", "<stdin>: ERROR: Expected identifier but found \"[\"\n")
 	expectParseErrorTS(t, "@new Function() class Foo {}", "<stdin>: ERROR: Expected identifier but found \"new\"\n")
 	expectParseErrorTS(t, "@() => {} class Foo {}", "<stdin>: ERROR: Unexpected \")\"\n")
 	expectParseErrorTS(t, "x = @y function() {}", "<stdin>: ERROR: Expected \"class\" but found \"function\"\n")
@@ -2101,7 +2109,9 @@ func TestTSDecorators(t *testing.T) {
 
 	// TypeScript 5.0+ allows this but Babel doesn't. I believe this is a bug
 	// with TypeScript: https://github.com/microsoft/TypeScript/issues/55336
-	expectParseErrorTS(t, "class Foo { @x<{}>().y<[], () => {}>() z: any }", "<stdin>: ERROR: Expected identifier but found \".\"\n")
+	expectParseErrorTS(t, "class Foo { @x<{}>().y<[], () => {}>() z: any }",
+		"<stdin>: ERROR: JavaScript decorator syntax does not allow \".\" after a call expression\n"+
+			"<stdin>: NOTE: Wrap this decorator in parentheses to allow arbitrary expressions:\n")
 
 	errorText := "<stdin>: ERROR: Transforming JavaScript decorators to the configured target environment is not supported yet\n"
 	expectParseErrorWithUnsupportedFeaturesTS(t, compat.Decorators, "@dec class Foo {}", errorText)
@@ -2130,9 +2140,15 @@ func TestTSDecorators(t *testing.T) {
 	expectParseErrorTS(t, "@x export default abstract", "<stdin>: ERROR: Decorators are not valid here\n")
 	expectParseErrorTS(t, "@x export @y default class {}", "<stdin>: ERROR: Decorators are not valid here\n<stdin>: ERROR: Unexpected \"default\"\n")
 
-	// JavaScript decorators are not allowed on declared fields
+	// From the TypeScript team: "We do allow postfix ! because it's TypeScript only."
+	// https://github.com/microsoft/TypeScript/issues/57756
+	expectPrintedTS(t, "@x!.y!.z class Foo {}", "@x.y.z class Foo {\n}\n")
+
+	// JavaScript decorators are not allowed on declared or abstract fields
 	expectParseErrorTS(t, "class Foo { @(() => {}) declare foo: any; @(() => {}) bar: any }",
 		"<stdin>: ERROR: Decorators are not valid here\n")
+	expectParseErrorTS(t, "abstract class Foo { @(() => {}) abstract foo: any; @(() => {}) bar: any }",
+		"<stdin>: ERROR: Decorators are not valid here\n")
 }
 
 func TestTSTry(t *testing.T) {
diff -pruN 0.20.1-1/internal/js_printer/js_printer.go 0.20.2-1/internal/js_printer/js_printer.go
--- 0.20.1-1/internal/js_printer/js_printer.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/js_printer/js_printer.go	2024-03-14 19:49:11.000000000 +0000
@@ -1545,7 +1545,7 @@ func (p *printer) printRequireOrImportEx
 		defer p.printDotThenSuffix()
 	}
 
-	// Make sure the comma operator is propertly wrapped
+	// Make sure the comma operator is properly wrapped
 	if meta.ExportsRef != ast.InvalidRef && level >= js_ast.LComma {
 		p.print("(")
 		defer p.print(")")
diff -pruN 0.20.1-1/internal/linker/linker.go 0.20.2-1/internal/linker/linker.go
--- 0.20.1-1/internal/linker/linker.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/linker/linker.go	2024-03-14 19:49:11.000000000 +0000
@@ -3370,7 +3370,7 @@ type cssImportOrder struct {
 // The only exception to this is "@layer". Evaluating a CSS file multiple
 // times is sort of equivalent to evaluating it once at the first location
 // as far as "@layer" is concerned. So we may in some cases keep both the
-// first and and last locations and only write out the "@layer" information
+// first and last locations and only write out the "@layer" information
 // for the first location.
 func (c *linkerContext) findImportedFilesInCSSOrder(entryPoints []uint32) (order []cssImportOrder) {
 	var visit func(uint32, []uint32, []css_ast.ImportConditions, []ast.ImportRecord)
@@ -3778,7 +3778,7 @@ func importConditionsAreEqual(a []css_as
 // For "lib.css", the entry with the conditions [supports(display: flex)] should
 // make the entry with the conditions [supports(display: flex), screen] redundant.
 //
-// Note that all of this deliberately ignores the existance of "@layer" because
+// Note that all of this deliberately ignores the existence of "@layer" because
 // that is handled separately. All of this is only for handling unlayered styles.
 func isConditionalImportRedundant(earlier []css_ast.ImportConditions, later []css_ast.ImportConditions) bool {
 	if len(later) > len(earlier) {
@@ -4599,8 +4599,6 @@ func (c *linkerContext) generateCodeForF
 	r renamer.Renamer,
 	waitGroup *sync.WaitGroup,
 	partRange partRange,
-	entryBits helpers.BitSet,
-	chunkAbsDir string,
 	toCommonJSRef ast.Ref,
 	toESMRef ast.Ref,
 	runtimeRequireRef ast.Ref,
@@ -5549,8 +5547,6 @@ func (c *linkerContext) generateChunkJS(
 			r,
 			&waitGroup,
 			partRange,
-			chunk.entryBits,
-			chunkAbsDir,
 			toCommonJSRef,
 			toESMRef,
 			runtimeRequireRef,
diff -pruN 0.20.1-1/internal/resolver/resolver.go 0.20.2-1/internal/resolver/resolver.go
--- 0.20.1-1/internal/resolver/resolver.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/resolver/resolver.go	2024-03-14 19:49:11.000000000 +0000
@@ -328,6 +328,9 @@ func NewResolver(call config.APICall, fs
 			visited = make(map[string]bool)
 		}
 		if options.TSConfigPath != "" {
+			if r.log.Level <= logger.LevelDebug {
+				r.debugLogs = &debugLogs{what: fmt.Sprintf("Resolving tsconfig file %q", options.TSConfigPath)}
+			}
 			res.tsConfigOverride, err = r.parseTSConfig(options.TSConfigPath, visited)
 		} else {
 			source := logger.Source{
@@ -345,6 +348,8 @@ func NewResolver(call config.APICall, fs
 				r.log.AddError(nil, logger.Range{}, fmt.Sprintf("Cannot read file %q: %s",
 					PrettyPath(r.fs, logger.Path{Text: options.TSConfigPath, Namespace: "file"}), err.Error()))
 			}
+		} else {
+			r.flushDebugLogs(flushDueToSuccess)
 		}
 	}
 
@@ -1264,6 +1269,26 @@ func (r resolverQuery) parseTSConfigFrom
 					}
 					goto pnpError
 				} else if result.status == pnpSuccess {
+					// If Yarn PnP path resolution succeeded, run a custom abbreviated
+					// version of node's module resolution algorithm. The Yarn PnP
+					// specification says to use node's module resolution algorithm verbatim
+					// but that isn't what Yarn actually does. See this for more info:
+					// https://github.com/evanw/esbuild/issues/2473#issuecomment-1216774461
+					if entries, _, dirErr := r.fs.ReadDirectory(result.pkgDirPath); dirErr == nil {
+						if entry, _ := entries.Get("package.json"); entry != nil && entry.Kind(r.fs) == fs.FileEntry {
+							// Check the "exports" map
+							if packageJSON := r.parsePackageJSON(result.pkgDirPath); packageJSON != nil && packageJSON.exportsMap != nil {
+								if absolute, ok, _ := r.esmResolveAlgorithm(result.pkgIdent, "."+result.pkgSubpath, packageJSON, result.pkgDirPath, source.KeyPath.Text); ok {
+									base, err := r.parseTSConfig(absolute.Primary.Text, visited)
+									if result, shouldReturn := maybeFinishOurSearch(base, err, absolute.Primary.Text); shouldReturn {
+										return result
+									}
+								}
+								goto pnpError
+							}
+						}
+					}
+
 					// Continue with the module resolution algorithm from node.js
 					extends = r.fs.Join(result.pkgDirPath, result.pkgSubpath)
 				}
@@ -1402,8 +1427,12 @@ func (r resolverQuery) parseTSConfigFrom
 		// Suppress warnings about missing base config files inside "node_modules"
 	pnpError:
 		if !helpers.IsInsideNodeModules(source.KeyPath.Text) {
-			r.log.AddID(logger.MsgID_TSConfigJSON_Missing, logger.Warning, &tracker, extendsRange,
-				fmt.Sprintf("Cannot find base config file %q", extends))
+			var notes []logger.MsgData
+			if r.debugLogs != nil {
+				notes = r.debugLogs.notes
+			}
+			r.log.AddIDWithNotes(logger.MsgID_TSConfigJSON_Missing, logger.Warning, &tracker, extendsRange,
+				fmt.Sprintf("Cannot find base config file %q", extends), notes)
 		}
 
 		return nil
diff -pruN 0.20.1-1/internal/resolver/yarnpnp.go 0.20.2-1/internal/resolver/yarnpnp.go
--- 0.20.1-1/internal/resolver/yarnpnp.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/internal/resolver/yarnpnp.go	2024-03-14 19:49:11.000000000 +0000
@@ -204,7 +204,7 @@ func (r resolverQuery) resolveToUnqualif
 			}
 
 			// If parentLocator isn't in manifest.fallbackExclusionList, then
-			if set, _ := manifest.fallbackExclusionList[parentLocator.ident]; !set[parentLocator.reference] {
+			if set := manifest.fallbackExclusionList[parentLocator.ident]; !set[parentLocator.reference] {
 				// Let fallback be RESOLVE_VIA_FALLBACK(manifest, ident)
 				fallback, _ := r.resolveViaFallback(manifest, ident)
 
@@ -302,9 +302,7 @@ func (r resolverQuery) findLocator(manif
 	}
 
 	// The relative path must not start with ./; trim it if needed
-	if strings.HasPrefix(relativeUrl, "./") {
-		relativeUrl = relativeUrl[2:]
-	}
+	relativeUrl = strings.TrimPrefix(relativeUrl, "./")
 
 	// If relativeUrl matches manifest.ignorePatternData, then
 	if manifest.ignorePatternData != nil && manifest.ignorePatternData.MatchString(relativeUrl) {
@@ -370,7 +368,7 @@ func (r resolverQuery) resolveViaFallbac
 	// Let referenceOrAlias be the entry from manifest.fallbackPool referenced by ident
 	referenceOrAlias, ok = manifest.fallbackPool[ident]
 
-	// Return it immediatly, whether it's defined or not
+	// Return it immediately, whether it's defined or not
 	if r.debugLogs != nil {
 		if ok {
 			r.debugLogs.addNote(fmt.Sprintf("    Found fallback for %q in \"fallbackPool\": [%s, %s]", ident,
diff -pruN 0.20.1-1/lib/deno/mod.ts 0.20.2-1/lib/deno/mod.ts
--- 0.20.1-1/lib/deno/mod.ts	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/lib/deno/mod.ts	2024-03-14 19:49:11.000000000 +0000
@@ -192,8 +192,6 @@ type SpawnFn = (cmd: string, options: {
   read(): Promise<Uint8Array | null>
   close(): Promise<void> | void
   status(): Promise<{ code: number }>
-  unref(): void
-  ref(): void
 }
 
 // Deno ≥1.40
@@ -235,8 +233,6 @@ const spawnNew: SpawnFn = (cmd, { args,
       await child.status
     },
     status: () => child.status,
-    unref: () => child.unref(),
-    ref: () => child.ref(),
   }
 }
 
@@ -277,8 +273,6 @@ const spawnOld: SpawnFn = (cmd, { args,
       child.close()
     },
     status: () => child.status(),
-    unref: () => { },
-    ref: () => { },
   }
 }
 
@@ -334,20 +328,12 @@ const ensureServiceIsRunning = (): Promi
       })
       readMoreStdout()
 
-      let refCount = 0
-      child.unref() // Allow Deno to exit when esbuild is running
-
-      const refs: common.Refs = {
-        ref() { if (++refCount === 1) child.ref(); },
-        unref() { if (--refCount === 0) child.unref(); },
-      }
-
       return {
         build: (options: types.BuildOptions) =>
           new Promise<types.BuildResult>((resolve, reject) => {
             service.buildOrContext({
               callName: 'build',
-              refs,
+              refs: null,
               options,
               isTTY,
               defaultWD,
@@ -359,7 +345,7 @@ const ensureServiceIsRunning = (): Promi
           new Promise<types.BuildContext>((resolve, reject) =>
             service.buildOrContext({
               callName: 'context',
-              refs,
+              refs: null,
               options,
               isTTY,
               defaultWD,
@@ -370,7 +356,7 @@ const ensureServiceIsRunning = (): Promi
           new Promise<types.TransformResult>((resolve, reject) =>
             service.transform({
               callName: 'transform',
-              refs,
+              refs: null,
               input,
               options: options || {},
               isTTY,
@@ -403,7 +389,7 @@ const ensureServiceIsRunning = (): Promi
           new Promise((resolve, reject) =>
             service.formatMessages({
               callName: 'formatMessages',
-              refs,
+              refs: null,
               messages,
               options,
               callback: (err, res) => err ? reject(err) : resolve(res!),
@@ -413,7 +399,7 @@ const ensureServiceIsRunning = (): Promi
           new Promise((resolve, reject) =>
             service.analyzeMetafile({
               callName: 'analyzeMetafile',
-              refs,
+              refs: null,
               metafile: typeof metafile === 'string' ? metafile : JSON.stringify(metafile),
               options,
               callback: (err, res) => err ? reject(err) : resolve(res!),
diff -pruN 0.20.1-1/lib/shared/types.ts 0.20.2-1/lib/shared/types.ts
--- 0.20.1-1/lib/shared/types.ts	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/lib/shared/types.ts	2024-03-14 19:49:11.000000000 +0000
@@ -664,16 +664,40 @@ export let version: string
 
 // Call this function to terminate esbuild's child process. The child process
 // is not terminated and re-created after each API call because it's more
-// efficient to keep it around when there are multiple API calls. This child
-// process normally exits automatically when the parent process exits, so you
-// usually don't need to call this function.
-//
-// One reason you might want to call this is if you know you will not make any
-// more esbuild API calls and you want to clean up resources (since the esbuild
-// child process takes up some memory even when idle).
+// efficient to keep it around when there are multiple API calls.
+//
+// In node this happens automatically before the parent node process exits. So
+// you only need to call this if you know you will not make any more esbuild
+// API calls and you want to clean up resources.
+//
+// Unlike node, Deno lacks the necessary APIs to clean up child processes
+// automatically. You must manually call stop() in Deno when you're done
+// using esbuild or Deno will continue running forever.
 //
 // Another reason you might want to call this is if you are using esbuild from
 // within a Deno test. Deno fails tests that create a child process without
 // killing it before the test ends, so you have to call this function (and
 // await the returned promise) in every Deno test that uses esbuild.
 export declare function stop(): Promise<void>
+
+// Note: These declarations exist to avoid type errors when you omit "dom" from
+// "lib" in your "tsconfig.json" file. TypeScript confusingly declares the
+// global "WebAssembly" type in "lib.dom.d.ts" even though it has nothing to do
+// with the browser DOM and is present in many non-browser JavaScript runtimes
+// (e.g. node and deno). Declaring it here allows esbuild's API to be used in
+// these scenarios.
+//
+// There's an open issue about getting this problem corrected (although these
+// declarations will need to remain even if this is fixed for backward
+// compatibility with older TypeScript versions):
+//
+//   https://github.com/microsoft/TypeScript-DOM-lib-generator/issues/826
+//
+declare global {
+  namespace WebAssembly {
+    interface Module {
+    }
+  }
+  interface URL {
+  }
+}
diff -pruN 0.20.1-1/lib/tsconfig-nolib.json 0.20.2-1/lib/tsconfig-nolib.json
--- 0.20.1-1/lib/tsconfig-nolib.json	1970-01-01 00:00:00.000000000 +0000
+++ 0.20.2-1/lib/tsconfig-nolib.json	2024-03-14 19:49:11.000000000 +0000
@@ -0,0 +1,11 @@
+{
+  "compilerOptions": {
+    "module": "CommonJS", // Allow the "import assignment" syntax
+    "target": "es2017", // Allow calling APIs such as "Object.entries"
+    "strict": true,
+    "lib": [], // Omit "dom" to test what happens with the "WebAssembly" type, which is defined in "dom"
+  },
+  "include": [
+    "shared/types.ts",
+  ],
+}
diff -pruN 0.20.1-1/npm/@esbuild/aix-ppc64/package.json 0.20.2-1/npm/@esbuild/aix-ppc64/package.json
--- 0.20.1-1/npm/@esbuild/aix-ppc64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/aix-ppc64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/aix-ppc64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The IBM AIX PowerPC 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/android-arm/package.json 0.20.2-1/npm/@esbuild/android-arm/package.json
--- 0.20.1-1/npm/@esbuild/android-arm/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/android-arm/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/android-arm",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "A WebAssembly shim for esbuild on Android ARM.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/android-arm64/package.json 0.20.2-1/npm/@esbuild/android-arm64/package.json
--- 0.20.1-1/npm/@esbuild/android-arm64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/android-arm64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/android-arm64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Android ARM 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/android-x64/package.json 0.20.2-1/npm/@esbuild/android-x64/package.json
--- 0.20.1-1/npm/@esbuild/android-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/android-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/android-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "A WebAssembly shim for esbuild on Android x64.",
   "repository": "https://github.com/evanw/esbuild",
   "license": "MIT",
diff -pruN 0.20.1-1/npm/@esbuild/darwin-arm64/package.json 0.20.2-1/npm/@esbuild/darwin-arm64/package.json
--- 0.20.1-1/npm/@esbuild/darwin-arm64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/darwin-arm64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/darwin-arm64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The macOS ARM 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/darwin-x64/package.json 0.20.2-1/npm/@esbuild/darwin-x64/package.json
--- 0.20.1-1/npm/@esbuild/darwin-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/darwin-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/darwin-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The macOS 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/freebsd-arm64/package.json 0.20.2-1/npm/@esbuild/freebsd-arm64/package.json
--- 0.20.1-1/npm/@esbuild/freebsd-arm64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/freebsd-arm64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/freebsd-arm64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The FreeBSD ARM 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/freebsd-x64/package.json 0.20.2-1/npm/@esbuild/freebsd-x64/package.json
--- 0.20.1-1/npm/@esbuild/freebsd-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/freebsd-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/freebsd-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The FreeBSD 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-arm/package.json 0.20.2-1/npm/@esbuild/linux-arm/package.json
--- 0.20.1-1/npm/@esbuild/linux-arm/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-arm/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-arm",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux ARM binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-arm64/package.json 0.20.2-1/npm/@esbuild/linux-arm64/package.json
--- 0.20.1-1/npm/@esbuild/linux-arm64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-arm64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-arm64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux ARM 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-ia32/package.json 0.20.2-1/npm/@esbuild/linux-ia32/package.json
--- 0.20.1-1/npm/@esbuild/linux-ia32/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-ia32/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-ia32",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux 32-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-loong64/package.json 0.20.2-1/npm/@esbuild/linux-loong64/package.json
--- 0.20.1-1/npm/@esbuild/linux-loong64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-loong64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-loong64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux LoongArch 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-mips64el/package.json 0.20.2-1/npm/@esbuild/linux-mips64el/package.json
--- 0.20.1-1/npm/@esbuild/linux-mips64el/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-mips64el/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-mips64el",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux MIPS 64-bit Little Endian binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-ppc64/package.json 0.20.2-1/npm/@esbuild/linux-ppc64/package.json
--- 0.20.1-1/npm/@esbuild/linux-ppc64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-ppc64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-ppc64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux PowerPC 64-bit Little Endian binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-riscv64/package.json 0.20.2-1/npm/@esbuild/linux-riscv64/package.json
--- 0.20.1-1/npm/@esbuild/linux-riscv64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-riscv64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-riscv64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux RISC-V 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-s390x/package.json 0.20.2-1/npm/@esbuild/linux-s390x/package.json
--- 0.20.1-1/npm/@esbuild/linux-s390x/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-s390x/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-s390x",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux IBM Z 64-bit Big Endian binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/linux-x64/package.json 0.20.2-1/npm/@esbuild/linux-x64/package.json
--- 0.20.1-1/npm/@esbuild/linux-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/linux-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/linux-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Linux 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/netbsd-x64/package.json 0.20.2-1/npm/@esbuild/netbsd-x64/package.json
--- 0.20.1-1/npm/@esbuild/netbsd-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/netbsd-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/netbsd-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The NetBSD AMD64 binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/openbsd-x64/package.json 0.20.2-1/npm/@esbuild/openbsd-x64/package.json
--- 0.20.1-1/npm/@esbuild/openbsd-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/openbsd-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/openbsd-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The OpenBSD 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/sunos-x64/package.json 0.20.2-1/npm/@esbuild/sunos-x64/package.json
--- 0.20.1-1/npm/@esbuild/sunos-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/sunos-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/sunos-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The illumos 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/win32-arm64/package.json 0.20.2-1/npm/@esbuild/win32-arm64/package.json
--- 0.20.1-1/npm/@esbuild/win32-arm64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/win32-arm64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/win32-arm64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Windows ARM 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/win32-ia32/package.json 0.20.2-1/npm/@esbuild/win32-ia32/package.json
--- 0.20.1-1/npm/@esbuild/win32-ia32/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/win32-ia32/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/win32-ia32",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Windows 32-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/@esbuild/win32-x64/package.json 0.20.2-1/npm/@esbuild/win32-x64/package.json
--- 0.20.1-1/npm/@esbuild/win32-x64/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/@esbuild/win32-x64/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "@esbuild/win32-x64",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The Windows 64-bit binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/npm/esbuild/package.json 0.20.2-1/npm/esbuild/package.json
--- 0.20.1-1/npm/esbuild/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/esbuild/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "esbuild",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "An extremely fast JavaScript and CSS bundler and minifier.",
   "repository": {
     "type": "git",
@@ -18,29 +18,29 @@
     "esbuild": "bin/esbuild"
   },
   "optionalDependencies": {
-    "@esbuild/aix-ppc64": "0.20.1",
-    "@esbuild/android-arm": "0.20.1",
-    "@esbuild/android-arm64": "0.20.1",
-    "@esbuild/android-x64": "0.20.1",
-    "@esbuild/darwin-arm64": "0.20.1",
-    "@esbuild/darwin-x64": "0.20.1",
-    "@esbuild/freebsd-arm64": "0.20.1",
-    "@esbuild/freebsd-x64": "0.20.1",
-    "@esbuild/linux-arm": "0.20.1",
-    "@esbuild/linux-arm64": "0.20.1",
-    "@esbuild/linux-ia32": "0.20.1",
-    "@esbuild/linux-loong64": "0.20.1",
-    "@esbuild/linux-mips64el": "0.20.1",
-    "@esbuild/linux-ppc64": "0.20.1",
-    "@esbuild/linux-riscv64": "0.20.1",
-    "@esbuild/linux-s390x": "0.20.1",
-    "@esbuild/linux-x64": "0.20.1",
-    "@esbuild/netbsd-x64": "0.20.1",
-    "@esbuild/openbsd-x64": "0.20.1",
-    "@esbuild/sunos-x64": "0.20.1",
-    "@esbuild/win32-arm64": "0.20.1",
-    "@esbuild/win32-ia32": "0.20.1",
-    "@esbuild/win32-x64": "0.20.1"
+    "@esbuild/aix-ppc64": "0.20.2",
+    "@esbuild/android-arm": "0.20.2",
+    "@esbuild/android-arm64": "0.20.2",
+    "@esbuild/android-x64": "0.20.2",
+    "@esbuild/darwin-arm64": "0.20.2",
+    "@esbuild/darwin-x64": "0.20.2",
+    "@esbuild/freebsd-arm64": "0.20.2",
+    "@esbuild/freebsd-x64": "0.20.2",
+    "@esbuild/linux-arm": "0.20.2",
+    "@esbuild/linux-arm64": "0.20.2",
+    "@esbuild/linux-ia32": "0.20.2",
+    "@esbuild/linux-loong64": "0.20.2",
+    "@esbuild/linux-mips64el": "0.20.2",
+    "@esbuild/linux-ppc64": "0.20.2",
+    "@esbuild/linux-riscv64": "0.20.2",
+    "@esbuild/linux-s390x": "0.20.2",
+    "@esbuild/linux-x64": "0.20.2",
+    "@esbuild/netbsd-x64": "0.20.2",
+    "@esbuild/openbsd-x64": "0.20.2",
+    "@esbuild/sunos-x64": "0.20.2",
+    "@esbuild/win32-arm64": "0.20.2",
+    "@esbuild/win32-ia32": "0.20.2",
+    "@esbuild/win32-x64": "0.20.2"
   },
   "license": "MIT"
 }
diff -pruN 0.20.1-1/npm/esbuild-wasm/package.json 0.20.2-1/npm/esbuild-wasm/package.json
--- 0.20.1-1/npm/esbuild-wasm/package.json	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/npm/esbuild-wasm/package.json	2024-03-14 19:49:11.000000000 +0000
@@ -1,6 +1,6 @@
 {
   "name": "esbuild-wasm",
-  "version": "0.20.1",
+  "version": "0.20.2",
   "description": "The cross-platform WebAssembly binary for esbuild, a JavaScript bundler.",
   "repository": {
     "type": "git",
diff -pruN 0.20.1-1/pkg/api/api_impl.go 0.20.2-1/pkg/api/api_impl.go
--- 0.20.1-1/pkg/api/api_impl.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/pkg/api/api_impl.go	2024-03-14 19:49:11.000000000 +0000
@@ -276,25 +276,6 @@ func validateLoader(value Loader) config
 	}
 }
 
-func validateEngine(value EngineName) compat.Engine {
-	switch value {
-	case EngineChrome:
-		return compat.Chrome
-	case EngineEdge:
-		return compat.Edge
-	case EngineFirefox:
-		return compat.Firefox
-	case EngineIOS:
-		return compat.IOS
-	case EngineNode:
-		return compat.Node
-	case EngineSafari:
-		return compat.Safari
-	default:
-		panic("Invalid loader")
-	}
-}
-
 var versionRegex = regexp.MustCompile(`^([0-9]+)(?:\.([0-9]+))?(?:\.([0-9]+))?(-[A-Za-z0-9]+(?:\.[A-Za-z0-9]+)*)?$`)
 
 func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.JSFeature, compat.CSSFeature, map[css_ast.D]compat.CSSPrefix, string) {
@@ -454,37 +435,6 @@ func validateExternals(log logger.Log, f
 	return result
 }
 
-func esmParsePackageName(packageSpecifier string) (packageName string, packageSubpath string, ok bool) {
-	if packageSpecifier == "" {
-		return
-	}
-
-	slash := strings.IndexByte(packageSpecifier, '/')
-	if !strings.HasPrefix(packageSpecifier, "@") {
-		if slash == -1 {
-			slash = len(packageSpecifier)
-		}
-		packageName = packageSpecifier[:slash]
-	} else {
-		if slash == -1 {
-			return
-		}
-		slash2 := strings.IndexByte(packageSpecifier[slash+1:], '/')
-		if slash2 == -1 {
-			slash2 = len(packageSpecifier[slash+1:])
-		}
-		packageName = packageSpecifier[:slash+1+slash2]
-	}
-
-	if strings.HasPrefix(packageName, ".") || strings.ContainsAny(packageName, "\\%") {
-		return
-	}
-
-	packageSubpath = "." + packageSpecifier[len(packageName):]
-	ok = true
-	return
-}
-
 func validateAlias(log logger.Log, fs fs.FS, alias map[string]string) map[string]string {
 	valid := make(map[string]string, len(alias))
 
diff -pruN 0.20.1-1/pkg/api/serve_other.go 0.20.2-1/pkg/api/serve_other.go
--- 0.20.1-1/pkg/api/serve_other.go	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/pkg/api/serve_other.go	2024-03-14 19:49:11.000000000 +0000
@@ -113,7 +113,7 @@ func (h *apiHandler) ServeHTTP(res http.
 	maybeWriteResponseBody := func(bytes []byte) { res.Write(bytes) }
 	isHEAD := req.Method == "HEAD"
 	if isHEAD {
-		maybeWriteResponseBody = func(bytes []byte) { res.Write(nil) }
+		maybeWriteResponseBody = func([]byte) { res.Write(nil) }
 	}
 
 	// Handle GET and HEAD requests
diff -pruN 0.20.1-1/scripts/deno-tests.js 0.20.2-1/scripts/deno-tests.js
--- 0.20.1-1/scripts/deno-tests.js	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/scripts/deno-tests.js	2024-03-14 19:49:11.000000000 +0000
@@ -15,22 +15,20 @@ try {
 Deno.mkdirSync(rootTestDir, { recursive: true })
 
 function test(name, backends, fn) {
-  const singleTest = (name, fn) => Deno.test({
-    name,
-    fn: () => new Promise((resolve, reject) => {
-      const minutes = 5
-      const timeout = setTimeout(() => reject(new Error(`Timeout for "${name}" after ${minutes} minutes`)), minutes * 60 * 1000)
-      const cancel = () => clearTimeout(timeout)
-      const promise = fn()
-      promise.then(cancel, cancel)
-      promise.then(resolve, reject)
-    }),
-  })
+  // Note: Do not try to add a timeout for the tests below. It turns out that
+  // calling "setTimeout" from within "Deno.test" changes how Deno waits for
+  // promises in a way that masks problems that would otherwise occur. We want
+  // test coverage for the way that people will likely be using these API calls.
+  //
+  // Specifically tests that Deno would otherwise have failed with "error:
+  // Promise resolution is still pending but the event loop has already
+  // resolved" were being allowed to pass instead. See this issue for more
+  // information: https://github.com/evanw/esbuild/issues/3682
 
   for (const backend of backends) {
     switch (backend) {
       case 'native':
-        singleTest(name + '-native', async () => {
+        Deno.test(name + '-native', async () => {
           let testDir = path.join(rootTestDir, name + '-native')
           await Deno.mkdir(testDir, { recursive: true })
           try {
@@ -43,7 +41,7 @@ function test(name, backends, fn) {
         break
 
       case 'wasm-main':
-        singleTest(name + '-wasm-main', async () => {
+        Deno.test(name + '-wasm-main', async () => {
           let testDir = path.join(rootTestDir, name + '-wasm-main')
           await esbuildWASM.initialize({ wasmModule, worker: false })
           await Deno.mkdir(testDir, { recursive: true })
@@ -57,7 +55,7 @@ function test(name, backends, fn) {
         break
 
       case 'wasm-worker':
-        singleTest(name + '-wasm-worker', async () => {
+        Deno.test(name + '-wasm-worker', async () => {
           let testDir = path.join(rootTestDir, name + '-wasm-worker')
           await esbuildWASM.initialize({ wasmModule, worker: true })
           await Deno.mkdir(testDir, { recursive: true })
diff -pruN 0.20.1-1/scripts/end-to-end-tests.js 0.20.2-1/scripts/end-to-end-tests.js
--- 0.20.1-1/scripts/end-to-end-tests.js	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/scripts/end-to-end-tests.js	2024-03-14 19:49:11.000000000 +0000
@@ -3528,6 +3528,43 @@ for (const minify of [[], ['--minify-syn
       `,
     }),
   )
+
+  // https://github.com/evanw/esbuild/issues/3700
+  tests.push(
+    test(['in.js', '--bundle', '--outfile=node.js'].concat(minify), {
+      'in.js': `
+        import imported from './data.json'
+        const native = JSON.parse(\`{
+          "hello": "world",
+          "__proto__": {
+            "sky": "universe"
+          }
+        }\`)
+        const literal1 = {
+          "hello": "world",
+          "__proto__": {
+            "sky": "universe"
+          }
+        }
+        const literal2 = {
+          "hello": "world",
+          ["__proto__"]: {
+            "sky": "universe"
+          }
+        }
+        if (Object.getPrototypeOf(native)?.sky) throw 'fail: native'
+        if (!Object.getPrototypeOf(literal1)?.sky) throw 'fail: literal1'
+        if (Object.getPrototypeOf(literal2)?.sky) throw 'fail: literal2'
+        if (Object.getPrototypeOf(imported)?.sky) throw 'fail: imported'
+      `,
+      'data.json': `{
+        "hello": "world",
+        "__proto__": {
+          "sky": "universe"
+        }
+      }`,
+    }),
+  )
 }
 
 // Test minification of top-level symbols
diff -pruN 0.20.1-1/scripts/ts-type-tests.js 0.20.2-1/scripts/ts-type-tests.js
--- 0.20.1-1/scripts/ts-type-tests.js	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/scripts/ts-type-tests.js	2024-03-14 19:49:11.000000000 +0000
@@ -379,11 +379,7 @@ async function main() {
   const tsc = path.join(__dirname, 'node_modules', 'typescript', 'lib', 'tsc.js')
   const esbuild_d_ts = path.join(testDir, 'node_modules', 'esbuild', 'index.d.ts')
   fs.mkdirSync(path.dirname(esbuild_d_ts), { recursive: true })
-  fs.writeFileSync(esbuild_d_ts, `
-    declare module 'esbuild' {
-      ${types.replace(/export declare/g, 'export')}
-    }
-  `)
+  fs.writeFileSync(esbuild_d_ts, types)
   let allTestsPassed = true
 
   // Check tests without errors
diff -pruN 0.20.1-1/version.txt 0.20.2-1/version.txt
--- 0.20.1-1/version.txt	2024-02-19 06:37:33.000000000 +0000
+++ 0.20.2-1/version.txt	2024-03-14 19:49:11.000000000 +0000
@@ -1 +1 @@
-0.20.1
+0.20.2
